uniformize <- function(x, x0, inverse=FALSE,smooth=FALSE,w,h) {
# this function implements transformation (35) and its inverse
# x is really the Y data in regression
# x0 can be vector! 
  n <- length(x)
  if(smooth){
	  if(missing(w)) {
		x <- sort(x)
		w <- rep(1/n,n)
		} else {
			if(any(w < 0)) warning("Negative weight encountered.")
			ord <- order(x)
			x <- x[ord]
			w <- w[ord]
		}	
    G=function(t,h){
      pnorm(t, mean = 0, sd = h)*(-2*h<t)*(t<2*h)+(t>=2*h)
    }
	  
	  if(inverse) {
	    wtemp=w
	    for(i in 1:n){
	      wtemp[i] <-sum(w*G(x[i]-x,h))/sum(w)
	    }
	    lwr <-max(2* x[1] - x[2],x[1]-h)
	    upr <-min(2* x[n] - x[n-1],x[n]+h)
	    Finv <- approxfun(c(0,wtemp,1),c(lwr,x,upr) )
	    return(Finv(x0))
	  } else {
	    return(sum(w*G(x0-x,h))/sum(w))
		}

  }

	else{	
	  if(missing(w)) {
			x <- sort(x)
			w <- (0:n)/n
		} else {
			if(any(w < 0)) warning("Negative weight encountered.")
			if(sum(w) != 1) { 
				w <- w/sum(w)
				warning("Weights do not sum to 1.  Renormalizing.")
			}
			ord <- order(x)
			x <- x[ord]
			w <- cumsum(c(0,w[ord]))
		}
	
	midpoints <- (x[-n] + x[-1])/2

	lwr <- min(2* midpoints[1] - midpoints[2], x[1] - (midpoints[1]-x[1]))
	upr <- max(2* midpoints[n-1] - midpoints[n-2], x[n] + (x[n] - midpoints[n-1]))
	midpoints <- c(lwr, midpoints, upr)
	
	if(inverse) {
		Finv <- approxfun(w, midpoints)
		return(Finv(x0))
	} else {
		Fmid <- approxfun(midpoints, w)
		F <- function(t) ifelse(t <= midpoints[1], 0, 
			ifelse(t >= midpoints[n+1], 1, Fmid(t)))
		return(F(x0))
	}
}
}

###########################################################################
getU<- function(x,y,ban=0, delete=FALSE,smooth=FALSE, edgecorrect=TRUE){ 
	# get the transformed data in MF2 using normal kernel
	# delete=F gives the regular u's of eq. (35) 
	# delete=T gives the predictive u's from the delete-1 data--eq. (39)
	# if edgecorrect=TRUE, leave out the u's that are too close to the edges
	n<- length(x) 
	if (ban==0) {ban<- crossval(x,y)}
  h=(.37*ban)^2
	u<- x 

	if (delete==FALSE){
	for (i in 1:n){ # this computes the u_t of eq. (35) 
	 w<-  dnorm((x-x[i]), mean = 0, sd = (0.37*ban))
	 u[i]<-  uniformize(y, y[i],  inverse=FALSE, smooth,w=w,h=h) } 
	}
  
  

	if (delete==TRUE){
	for(k in 1:n) {
	xx <- x[-k]
	yy <- y[-k]
	 w<-  dnorm((xx-x[k]), mean = 0, sd = (0.37*ban))
	  if(sum(w==rep(0,n-1))==n-1)
		u[k]=NA
	  else{
      u[k]<-  uniformize(yy, y[k],  inverse=FALSE, smooth,w=w,h=h)
	  }
	 }
	} # end if delete==TRUE

	if (edgecorrect==TRUE){
	xmin<- min(x, na.rm=T)
	xmax<- max(x, na.rm=T)
  u[ abs(x-xmin) < ban/2 | abs(x-xmax) < ban/2 ]<- NA*u[ abs(x-xmin) < ban/2 | abs(x-xmax) < ban/2 ]
  }

	na.rm(u)}
	##################################
crossval<- function(x,y, nstep=20,bmax=0,  L1=FALSE){
		# finds the best bandwidth for ksmooth by cross-validation
		# searches for bandwidts from zero to bmax 
		# uses criterion L1 or L2 (default)
		# nstep is number of grid points for bandwidth search
		# stud=TRUE means studentize by kernel smoothed estimate of variance 
			# (with same bandwidth)

	if (bmax==0){bmax<- 1.5*sqrt(var(x))}

	bstep<-  bmax/nstep
	n<- length(x)
	SSE<- c(1:nstep)*0 

	if(L1==FALSE){
		for (i in 1:nstep){
			for (k in 1:n){
				xx<- x[-k]
				yy<- y[-k]
		kss<- ksmooth(xx,yy,"normal",ban=(i*bstep),x.points=x[k])$y
			 SSE[i]<- SSE[i]+(y[k]-kss)^2
				}}}
				
	if(L1==TRUE){
	for (i in 1:nstep){
				for (k in 1:n){
					xx<- x[-k]
					yy<- y[-k]
					kss<- ksmooth(xx,yy,"normal",ban=(i*bstep),x.points=x[k])$y
					SSE[i]<- SSE[i]+ abs(y[k]-kss) 
			}
		}
	}
k<- c(1:nstep)*bstep
k<- k[SSE==min(SSE,na.rm = TRUE)]
na.rm(k)
}

##############################################
na.rm<-function(x)
{	x[!is.na(x)]
}

