#Get prediction interval of u[n] using local boostrap
lbsPI <- function(u,coverage=0.95,forward=TRUE){
  # u is a vector of time series
  # If set forward = FALSE, backward method would be applied
  n <- length(u)
  if((u[n]>mean(u)+2*sd(u))||(u[n]<mean(u)-2*sd(u))){
    stop("the last observation of the series is two standard deviation away from the rest of the data. Please retry with a new series")
  }
  
  x=u[1:(n-1)]
  y=u[2:n]
  
  gauss <- function(x) 1/sqrt(2*pi) * exp(-(x^2)/2)
  truncate=as.numeric(integrate(Vectorize(gauss),-5,5)[1])
  W=function(x){
    return ( gauss(x)/truncate*as.numeric(x<=5 & x>=-5))
  }  
  chooseh=function(x,y,nstep=20,bmax=0){
    if (bmax==0){bmax<-sd(x)}
    hp=bmax/nstep*1:nstep
    sumvar=rep(0,nstep)
    for(i in 1:nstep){
      for(j in 1:length(x)){
        sumvar[i]=(sum(gauss((x[j]-x[-j])/hp[i])*y[-j]/sum(gauss((x[j] - x[-j])/hp[i])))-y[j])^2+sumvar[i]
      }   
    }
    h=hp[order(sumvar)][1]
    return(h)
  }
  h=chooseh(x,y)
  fvhat=sum(W((u[n]-x)/h)*y/sum(W((u[n] - x)/h)))
  #########generate one-step ahead value ########################
  generate=function(x,y,condx){
    p=W((condx-x)/h)/sum(W((condx - x)/h))#transition density
    len=length(x)
    cp=rep(0,len)#cumulative transition density
    for (i in 1:len){
      cp[i]=sum(p[1:i])
      while(is.na(cp[i])) return(NA)
    }
    z=runif(1)
    j=1
    while(cp[j]<z) j=j+1
    return(y[j])
  }
  
  bh=chooseh(y,x)
  bgenerate=function(x,y,condx){
    p=W((condx-x)/bh)/sum(W((condx - x)/bh))#transition density
    len=length(x)
    cp=rep(0,len)#cumulative transition density
    for (i in 1:len){
      cp[i]=sum(p[1:i])
      while(is.na(cp[i])) return(NA)
    }
    z=runif(1)
    j=1
    while(cp[j]<z) j=j+1
    return(y[j])
  }
  
  ##################generate xstar forward#########################
  bootstrap=function(x,y){
    ustar=rep(0,n)
    ustar[1]=sample(u,1)
    for (i in 1:(n-1)){
      ustar[i+1]=generate(x,y,ustar[i])
    }
    fvstar=generate(x,y,u[n])
    list(ustar=ustar,fvstar=fvstar)
  }
  
  ##############generate xstar backward#####################
  bbootstrap=function(x,y){
    ustar=rep(0,n)
    ustar[n]=u[n]
    for (i in 1:(n-1)){
      ustar[n-i]=bgenerate(y,x,ustar[n-i+1])
      while(is.na(ustar[n-i])){
        if(i==1){ustar[n-1]=bgenerate(y,x,u[n])}
        if(i==2){
          ustar[n-1]=bgenerate(y,x,u[n])
          ustar[n-2]=bgenerate(y,x,ustar[n-1])                    
        }
        if(i>2){
          ustar[n-i+2]=bgenerate(y,x,ustar[n-i+3])
          ustar[n-i+1]=bgenerate(y,x,ustar[n-i+2])
          ustar[n-i]=bgenerate(y,x,ustar[n-i+1]) 
        }
      }
    }
    fvstar=generate(x,y,u[n])
    list(ustar=ustar,fvstar=fvstar)
  }
  
  
  if(forward){##forward method
    fvstarhat=rep(0,250)
    fvstar=rep(0,250)
    for (i in 1:250){
      result=bootstrap(x,y)
      ustar=result$ustar
      xstar=ustar[1:(n-1)]
      ystar=ustar[2:n]
      fvstarhat[i]=sum(W((u[n]-xstar)/h)*ystar/sum(W((u[n] - xstar)/h)))
      fvstar[i]=result$fvstar
    }
    interval=fvstar-fvstarhat
    interval=interval[!is.na(interval)]
    q=quantile(interval,c((1-coverage)/2,1-(1-coverage)/2))
    up=q[2]
    down=q[1]
  }else{#backward method
    bfvstarhat=rep(0,250)
    bfvstar=rep(0,250)
    for (i in 1:250){
      result=bbootstrap(x,y)
      ustar=result$ustar
      xstar=ustar[1:(n-1)]
      ystar=ustar[2:n]
      bfvstarhat[i]=sum(W((u[n]-xstar)/h)*ystar/sum(W((u[n] - xstar)/h)))
      bfvstar[i]=result$fvstar
    }
    binterval=bfvstar-bfvstarhat
    binterval=binterval[!is.na(binterval)]
    q=quantile(binterval,c((1-coverage)/2,1-(1-coverage)/2))
    up=q[2]
    down=q[1]
  }
  list(up=up+fvhat,down=down+fvhat)
}

#get prediction interval of u[n] based on the transition density(Rajarshi method)
RajPI <- function(u,coverage=0.95,forward=TRUE){
  n <- length(u)
  if((u[n]>mean(u)+2*sd(u))||(u[n]<mean(u)-2*sd(u))){
    stop("the last observation of the series is two standard deviation away from the rest of the data. Please retry with a new series")
  }
  x=u[1:(n-1)]
  y=u[2:(n)]
  
  ###############f(x,y)and f(x|y)#####################
  gauss <- function(x) 1/sqrt(2*pi) * exp(-(x^2)/2)
  h=.9*min(sd(u),(quantile(u)[4]-quantile(u)[2])/1.34)*n^(-1/4)  
  
  fx=function(a) sum (gauss((a - x)/h)/((n-1)*h))
  fxy=function(a,b) sum(gauss((a - x)/h)*gauss((b-y)/h)/((n-1) * h^2))
  f=function(a,b) fxy(a,b)/fx(a) #transition density f(b|a)
  
  ###############predicted value#####################
  minu=min(u)-3*h
  maxu=max(u)+3*h
  xf=function(y) y*f(u[n],y)
  fvhat=as.numeric(integrate(Vectorize(xf),minu,maxu)[1])
  
  #########generate x1, one-step ahead and back value ########################
  generatex1=function(){#generate x1 from fx()
    fy=sapply(x,fx)
    fmax=max(fy)
    xsample=runif(1,minu,maxu)
    ysample <- runif(1, 0, fmax)
    while(fx(xsample) <ysample){
      xsample=runif(1,minu,maxu)
      ysample <- runif(1, 0, fmax)
    }
    return(xsample)
  }
  
  generate=function(f,condx){#forward,generate one step ahead pseudo data from f(y|condx)
    ff=function(y) f(condx,y) 
    fy=sapply(u,ff)
    fmax=max(fy)
    xsample=runif(1,minu,maxu)
    ysample <- runif(1, 0, fmax)
    while(ff(xsample) <ysample){
      xsample=runif(1,minu,maxu)
      ysample  <- runif(1, 0, fmax)
    }
    return(xsample)
  }
  
  ##################generate xstar forward#########################
  bootstrap=function(start){
    ustar=rep(0,n)
    ustar[1]=start
    for (i in 1:(n-1)){
      ustar[i+1]=generate(f,ustar[i])
    }
    list(ustar=ustar)  
  }
  ##############generate xstar backward#####################
  bx=u[n:2]
  by=u[(n-1):1]
  bfx=function(a) sum(gauss((a - bx)/h)/((n-1)*h))
  bfxy=function(a,b) sum(gauss((a - bx)/h)*gauss((b-by)/h)/((n-1) * h^2))
  bf=function(a,b) bfxy(a,b)/bfx(a) #bf(b|a)
  
  bbootstrap=function(){
    ustar=rep(0,n)
    ustar[n]=u[n]
    for (i in 1:(n-1)){
      ustar[n-i]=generate(bf,ustar[n-i+1])
    }
    list(ustar=ustar)
  }
 
  if(forward){
    fvstarhat=rep(0,250)
    fvstar=rep(0,250)
    for (i in 1:250){
      start=generatex1()
      ustar=bootstrap(start)$ustar
      xstar=ustar[1:(n-1)]
      ystar=ustar[2:n]
      fxstar=function(a) sum(gauss((a - xstar)/h)/((n-1)*h))
      fxystar=function(a,b) sum(gauss((a - xstar)/h)*gauss((b-ystar)/h)/((n-1) * h^2))
      fstar=function(a,b) fxystar(a,b)/fxstar(a) #f*(b|a)
      xfstar=function(y) y*fstar(u[n],y)
      fvstarhat[i]=as.numeric(integrate(Vectorize(xfstar),minu,maxu)[1])
      fvstar[i]=generate(f,u[n])
    }
    interval=fvstar-fvstarhat
    q=quantile(interval,c((1-coverage)/2,1-(1-coverage)/2))
    up=q[2]
    down=q[1]
  }else{
    bfvstarhat=rep(0,250)
    fvstar=rep(0,250)
    for (i in 1:250){
      ustar=bbootstrap()$ustar
      xstar=ustar[1:(n-1)]
      ystar=ustar[2:n]
      fxstar=function(a) sum(gauss((a - xstar)/h)/((n-1)*h))
      fxystar=function(a,b) sum(gauss((a - xstar)/h)*gauss((b-ystar)/h)/((n-1) * h^2))
      fstar=function(a,b) fxystar(a,b)/fxstar(a) #f*(b|a)
      xfstar=function(y) y*fstar(u[n],y)
      bfvstarhat[i]=as.numeric(integrate(Vectorize(xfstar),minu,maxu)[1])
      fvstar[i]=generate(f,u[n])
    }
    binterval=fvstar-bfvstarhat
    q=quantile(binterval,c((1-coverage)/2,1-(1-coverage)/2))
    up=q[2]
    down=q[1]
  }
  list(up=up+fvhat,down=down+fvhat)    
}

#Simple Model Free
MFPI <- function(u,coverage=0.95,predictive=TRUE){
  uniformize <- function(x, x0, inverse=FALSE, w) {
    # this function implements transformation (35) and its inverse
    # IF inverse=FALSE, then x is really the Y data in regression
    # IF inverse=TRUE, then x is actually the x variable
    # x0 can be vector! 
    n <- length(x)
    if(missing(w)) {
      x <- sort(x)
      w <- (0:n)/n
    } else {
      if(any(w < 0)) warning("Negative weight encountered.")
      if(sum(w) != 1) { 
        w <- w/sum(w)
        warning("Weights do not sum to 1.  Renormalizing.")
      }
      ord <- order(x)
      x <- x[ord]
      w <- cumsum(c(0,w[ord]))
    }
    
    midpoints <- (x[-n] + x[-1])/2
    
    lwr <- min(2* midpoints[1] - midpoints[2], x[1] - (midpoints[1]-x[1]))
    upr <- max(2* midpoints[n-1] - midpoints[n-2], x[n] + (x[n] - midpoints[n-1]))
    midpoints <- c(lwr, midpoints, upr)
    
    if(inverse) {
      Finv <- approxfun(w, midpoints)
      return(Finv(x0))
    } else {
      Fmid <- approxfun(midpoints, w)
      F <- function(t) ifelse(t <= midpoints[1], 0, 
                              ifelse(t >= midpoints[n+1], 1, Fmid(t)))
      return(F(x0))
    }
  }
  getU<- function(x,y,ban=0, delete=FALSE, edgecorrect=TRUE){ 
    # get the transformed data in MF2 using normal kernel
    # delete=F gives the regular u's of eq. (35) 
    # delete=T gives the predictive u's from the delete-1 data--eq. (39)
    # if edgecorrect=TRUE, leave out the u's that are too close to the edges
    n<- length(x) 
    if (ban==0) {ban<- crossval(x,y)}
    u<- x 
    
    if (delete==FALSE){
      for (i in 1:n){ # this computes the u_t of eq. (35) 
        w<-  dnorm((x-x[i]), mean = 0, sd = (0.37*ban))
        u[i]<-  uniformize(y, y[i],  inverse=FALSE, w=w) } }
    
    if (delete==TRUE){
      for(k in 1:n) {
        xx <- x[-k]
        yy <- y[-k]
        w<-  dnorm((xx-x[k]), mean = 0, sd = (0.37*ban))
        if(sum(w==rep(0,n-1))==n-1)#x[k] far from others
          u[k]=NA
        else{
          u[k]<-  uniformize(yy, y[k],  inverse=FALSE, w=w)
        }
      } 
    } # end if delete==TRUE
    
    if (edgecorrect==TRUE){
      xmin<- min(x, na.rm=T)
      xmax<- max(x, na.rm=T)
      u[ abs(x-xmin) < ban/2 | abs(x-xmax) < ban/2 ]<- NA*u[ abs(x-xmin) < ban/2 | abs(x-xmax) < ban/2 ]
    }
    
    na.rm(u)} 
  crossval<- function(x,y, nstep=20,bmax=0,  L1=FALSE){
    # finds the best bandwidth for ksmooth by cross-validation
    # searches for bandwidts from zero to bmax 
    # uses criterion L1 or L2 (default)
    # nstep is number of grid points for bandwidth search
    # stud=TRUE means studentize by kernel smoothed estimate of variance 
    # (with same bandwidth)
    
    if (bmax==0){bmax<- 1.5*sqrt(var(x))}
    
    bstep<-  bmax/nstep
    n<- length(x)
    SSE<- c(1:nstep)*0 
    
    if(L1==FALSE){
      for (i in 1:nstep){
        for (k in 1:n){
          xx<- x[-k]
          yy<- y[-k]
          kss<- ksmooth(xx,yy,"normal",ban=(i*bstep),x.points=x[k])$y 
          SSE[i]<- SSE[i]+(y[k]-kss )^2
        }}
      if(all(is.na(SSE))){
        SSE<- c(1:nstep)*0 
        for (i in 1:nstep){
          for (k in 1:n){
            xx<- x[-k]
            yy<- y[-k]
            kss<- ksmooth(xx,yy,"normal",ban=(i*bstep),x.points=x[k])$y 
            if(is.na(kss)) kss <- 0
            SSE[i]<- SSE[i]+(y[k]-kss )^2
          }}
      }
    }
    
    if(L1==TRUE){
      for (i in 1:nstep){
        for (k in 1:n){
          xx<- x[-k]
          yy<- y[-k]
          kss<- ksmooth(xx,yy,"normal",ban=(i*bstep),x.points=x[k])$y
          SSE[i]<- SSE[i]+ abs(y[k]-kss) 
        }
      }
      if(all(is.na(SSE))){
        SSE<- c(1:nstep)*0 
        for (i in 1:nstep){
          for (k in 1:n){
            xx<- x[-k]
            yy<- y[-k]
            kss<- ksmooth(xx,yy,"normal",ban=(i*bstep),x.points=x[k])$y
            if(is.na(kss)) kss<-0
            SSE[i]<- SSE[i]+ abs(y[k]-kss) 
          }
        }
      }
    }
    k<- c(1:nstep)*bstep
    k<- k[SSE==min(SSE,na.rm = TRUE)]
    na.rm(k)
  }
  na.rm<-function(x)
  {	x[!is.na(x)]
  }
  n <- length(u)
  x=u[1:(n-1)]
  y=u[2:(n)]
  b=crossval(x,y)
  if((u[n]>mean(u)+2*sd(u))||(u[n]<mean(u)-2*sd(u))||length(b)==0){
    stop("the last observation of the series is two standard deviation away from the rest of the data. Please retry with a new series")
  }
  boots=function(u,v){
    vstar=sample(v,n+100,replace=TRUE)
    temp=rep(0,100+n)
    temp[1]=sample(u,1)
    for (i in 1:(n+99)){
      w=dnorm((x-temp[i]),mean=0,sd=(0.37*b))
      temp[i+1]=uniformize(y,vstar[i+1],inverse=TRUE,w=w)
    }
    ustar=temp[101:(100+n)]
    list(ustar=ustar,vstar=vstar[(101):(100+n)])
  }
  if(!predictive){
    v=getU(x,y,b)
    w=dnorm((x-u[n]),mean=0,sd=(0.37*b))
    yfhat=mean(uniformize(y,v,inverse=TRUE,w=w))
    wf=dnorm((x-u[n]),mean=0,sd=0.37*b)
    yfstar=rep(0,250)
    yfstarhat=rep(0,250)
    for (i in 1:250){
      yfstar[i]=uniformize(y,sample(v,1),inverse=TRUE,w=wf)
      result=boots(u,v)
      ustar=result$ustar
      xstar=ustar[1:(n-1)]
      ystar=ustar[2:n]
      vstar=result$vstar
      wfstar=dnorm((xstar-u[n]),mean=0,sd=0.37*b)
      yfstarhat[i]=mean(uniformize(ystar,vstar,inverse=TRUE,w=wfstar))
    }
    interval=yfstar-yfstarhat
    interval=interval[!is.na(interval)]
    q=quantile(interval,c((1-coverage)/2,1-(1-coverage)/2),na.rm=TRUE)
    up=q[2]+yfhat
    down=q[1]+yfhat
  }else{
    prev=getU(x,y,b,delete=TRUE)
    prev=prev[1-prev>10^(-6) & prev!=0]
    w=dnorm((x-u[n]),mean=0,sd=(0.37*b))
    yfhat1=mean(uniformize(y,prev,inverse=TRUE,w=w))
    wf=dnorm((x-u[n]),mean=0,sd=0.37*b)
    yfstar1=rep(0,250)
    yfstarhat1=rep(0,250)
    for (i in 1:250){
      yfstar1[i]=uniformize(y,sample(prev,1),inverse=TRUE,w=wf)
      result=boots(u,prev)
      ustar=result$ustar
      xstar=ustar[1:(n-1)]
      ystar=ustar[2:n]
      vstar=result$vstar
      wfstar=dnorm((xstar-u[n]),mean=0,sd=0.37*b)
      yfstarhat1[i]=mean(uniformize(ystar,vstar,inverse=TRUE,w=wfstar))
    }
    interval1=yfstar1-yfstarhat1
    interval1=interval1[!is.na(interval1)]
    q=quantile(interval1,c((1-coverage)/2,1-(1-coverage)/2),na.rm=TRUE)
    up=q[2]+yfhat1
    down=q[1]+yfhat1
  }
  
  list(up=up,down=down)
}

#Smooth Model Free
SMFPI <- function(u,coverage=0.95,predictive=TRUE){
  uniformize <- function(x, x0, inverse=FALSE,smooth=FALSE,w,h) {
    # this function implements transformation (35) and its inverse
    # x is really the Y data in regression
    # x0 can be vector! 
    n <- length(x)
    if(smooth){
      if(missing(w)) {
        x <- sort(x)
        w <- rep(1/n,n)
      } else {
        if(any(w < 0)) warning("Negative weight encountered.")
        ord <- order(x)
        x <- x[ord]
        w <- w[ord]
      }	
      G=function(t,h){
        pnorm(t, mean = 0, sd = h)*(-2*h<t)*(t<2*h)+(t>=2*h)
      }
      
      if(inverse) {
        wtemp=w
        for(i in 1:n){
          wtemp[i] <-sum(w*G(x[i]-x,h))/sum(w)
        }
        lwr <-max(2* x[1] - x[2],x[1]-h)
        upr <-min(2* x[n] - x[n-1],x[n]+h)
        Finv <- approxfun(c(0,wtemp,1),c(lwr,x,upr) )
        return(Finv(x0))
      } else {
        return(sum(w*G(x0-x,h))/sum(w))
      }
      
    }
    
    else{	
      if(missing(w)) {
        x <- sort(x)
        w <- (0:n)/n
      } else {
        if(any(w < 0)) warning("Negative weight encountered.")
        if(sum(w) != 1) { 
          w <- w/sum(w)
          warning("Weights do not sum to 1.  Renormalizing.")
        }
        ord <- order(x)
        x <- x[ord]
        w <- cumsum(c(0,w[ord]))
      }
      
      midpoints <- (x[-n] + x[-1])/2
      
      lwr <- min(2* midpoints[1] - midpoints[2], x[1] - (midpoints[1]-x[1]))
      upr <- max(2* midpoints[n-1] - midpoints[n-2], x[n] + (x[n] - midpoints[n-1]))
      midpoints <- c(lwr, midpoints, upr)
      
      if(inverse) {
        Finv <- approxfun(w, midpoints)
        return(Finv(x0))
      } else {
        Fmid <- approxfun(midpoints, w)
        F <- function(t) ifelse(t <= midpoints[1], 0, 
                                ifelse(t >= midpoints[n+1], 1, Fmid(t)))
        return(F(x0))
      }
    }
  }
  getU<- function(x,y,ban=0, delete=FALSE,smooth=FALSE, edgecorrect=TRUE){ 
    # get the transformed data in MF2 using normal kernel
    # delete=F gives the regular u's of eq. (35) 
    # delete=T gives the predictive u's from the delete-1 data--eq. (39)
    # if edgecorrect=TRUE, leave out the u's that are too close to the edges
    n<- length(x) 
    if (ban==0) {ban<- crossval(x,y)}
    h=(.37*ban)^2
    u<- x 
    
    if (delete==FALSE){
      for (i in 1:n){ # this computes the u_t of eq. (35) 
        w<-  dnorm((x-x[i]), mean = 0, sd = (0.37*ban))
        u[i]<-  uniformize(y, y[i],  inverse=FALSE, smooth,w=w,h=h) } 
    }
    
    
    
    if (delete==TRUE){
      for(k in 1:n) {
        xx <- x[-k]
        yy <- y[-k]
        w<-  dnorm((xx-x[k]), mean = 0, sd = (0.37*ban))
        if(sum(w==rep(0,n-1))==n-1)
          u[k]=NA
        else{
          u[k]<-  uniformize(yy, y[k],  inverse=FALSE, smooth,w=w,h=h)
        }
      }
    } # end if delete==TRUE
    
    if (edgecorrect==TRUE){
      xmin<- min(x, na.rm=T)
      xmax<- max(x, na.rm=T)
      u[ abs(x-xmin) < ban/2 | abs(x-xmax) < ban/2 ]<- NA*u[ abs(x-xmin) < ban/2 | abs(x-xmax) < ban/2 ]
    }
    
    na.rm(u)}
  crossval<- function(x,y, nstep=20,bmax=0,  L1=FALSE){
    # finds the best bandwidth for ksmooth by cross-validation
    # searches for bandwidts from zero to bmax 
    # uses criterion L1 or L2 (default)
    # nstep is number of grid points for bandwidth search
    # stud=TRUE means studentize by kernel smoothed estimate of variance 
    # (with same bandwidth)
    
    if (bmax==0){bmax<- 1.5*sqrt(var(x))}
    
    bstep<-  bmax/nstep
    n<- length(x)
    SSE<- c(1:nstep)*0 
    
    if(L1==FALSE){
      for (i in 1:nstep){
        for (k in 1:n){
          xx<- x[-k]
          yy<- y[-k]
          kss<- ksmooth(xx,yy,"normal",ban=(i*bstep),x.points=x[k])$y
          SSE[i]<- SSE[i]+(y[k]-kss)^2
        }}}
    
    if(L1==TRUE){
      for (i in 1:nstep){
        for (k in 1:n){
          xx<- x[-k]
          yy<- y[-k]
          kss<- ksmooth(xx,yy,"normal",ban=(i*bstep),x.points=x[k])$y
          SSE[i]<- SSE[i]+ abs(y[k]-kss) 
        }
      }
    }
    k<- c(1:nstep)*bstep
    k<- k[SSE==min(SSE,na.rm = TRUE)]
    na.rm(k)
  }
  na.rm<-function(x){	x[!is.na(x)]
  }
  n <- length(u)
  x=u[1:(n-1)]
  y=u[2:(n)]
  b=crossval(x,y)
  if((u[n]>mean(u)+2*sd(u))||(u[n]<mean(u)-2*sd(u))||length(b)==0){
    stop("the last observation of the series is two standard deviation away from the rest of the data. Please retry with a new series")
  }
  h=(b*.37)^2
  v=getU(x,y,b,smooth=TRUE)
  if(!predictive){
    w=dnorm((x-u[n]),mean=0,sd=(0.37*b))
    yfhat=mean(uniformize(y,v,inverse=TRUE,smooth=TRUE,w=w,h=h))
    boots=function(u,v){
      vstar=sample(v,n+100,replace=TRUE)
      temp=rep(0,100+n)
      temp[1]=sample(u,1)
      for (i in 1:(n+99)){
        w=dnorm((x-temp[i]),mean=0,sd=(0.37*b))
        temp[i+1]=uniformize(y,vstar[i+1],inverse=TRUE,smooth=TRUE,w=w,h=h)
      }
      ustar=temp[101:(100+n)]
      list(ustar=ustar,vstar=vstar[(101):(100+n)])
    }
    wf=dnorm((x-u[n]),mean=0,sd=0.37*b)
    yfstar=rep(0,250)
    yfstarhat=rep(0,250)
    for (i in 1:250){
      yfstar[i]=uniformize(y,sample(v,1),inverse=TRUE,smooth=TRUE,w=wf,h=h)
      result=boots(u,v)
      ustar=result$ustar
      xstar=ustar[1:(n-1)]
      ystar=ustar[2:n]
      vstar=result$vstar
      wfstar=dnorm((xstar-u[n]),mean=0,sd=0.37*b)
      yfstarhat[i]=mean(uniformize(ystar,vstar,inverse=TRUE,smooth=TRUE,w=wfstar,h=h))
    }
    interval=yfstar-yfstarhat
    interval=interval[!is.na(interval)]
    q=quantile(interval,c((1-coverage)/2,1-(1-coverage)/2))
    up=q[2]+yfhat
    down=q[1]+yfhat
  }else{
    prev=getU(x,y,b,delete=TRUE,smooth=TRUE)
    prev=prev[prev<1 & prev>0]
    yfhat1=mean(uniformize(y,prev,inverse=TRUE,smooth=TRUE,w=w,h=h))
    yfstar1=rep(0,250)
    yfstarhat1=rep(0,250)
    for (i in 1:250){
      yfstar1[i]=uniformize(y,sample(prev,1),inverse=TRUE,smooth=TRUE,w=wf,h=h)
      result=boots(u,prev)
      ustar=result$ustar
      xstar=ustar[1:(n-1)]
      ystar=ustar[2:n]
      vstar=result$vstar
      wfstar=dnorm((xstar-u[n]),mean=0,sd=0.37*b)
      yfstarhat1[i]=mean(uniformize(ystar,vstar,inverse=TRUE,smooth=TRUE,w=wfstar,h=h)) 
    }
    interval1=yfstar1-yfstarhat1
    interval1=interval1[!is.na(interval1)]
    q=quantile(interval1,c((1-coverage)/2,1-(1-coverage)/2))
    up=q[2]+yfhat1
    down=q[1]+yfhat1
  }
  list(up=up,down=down)
}
